#ifndef ARBREGEN_HPP
#define ARBREGEN_HPP

#include <vector>
#include "utils.PRO2"


template <class T> class ArbreGen {
private:
  struct node_arbreGen {
    T info;
    vector<node_arbreGen*> seg;
  };
  node_arbreGen* primer_node;
  

public:

  ArbreGen(){
  /* Pre: cert */
  /* Post: El resultat és un arbre sense cap element */
    primer_node= NULL;
  }
  
  ArbreGen(const T& x){
    /* Pre: cert */
    /* Post: El resultat és un arbre amb x com a arrel i sense fills */
    primer_node= new node_arbreGen;
    primer_node->info = x;
  }

  ArbreGen(const ArbreGen& original)
  /* Pre: cert */
  /* Post: el resultat és una còpia d'original */
  {
    primer_node = copia_node_arbreGen(original.primer_node);
  }


  // Destructora: 

  ~ArbreGen(){
    esborra_node_arbreGen(primer_node);
  }


  ArbreGen& operator=(const ArbreGen& original)
  /* Pre: cert */
  /* Post: El p.i. passa a ser una còpia d'original i qualsevol
     contingut anterior del p.i. ha estat esborrat
     (excepte si el p.i. i original ja eren el mateix objecte) */
  {
    if (this != &original) {
      esborra_node_arbreGen(primer_node);
      primer_node = copia_node_arbreGen(original.primer_node);
    }
    return *this;
  }


// Modificadores


  void a_buit()
  /* Pre: cert */
  /* Post: el p.i. és un arbre buit i qualsevol
     contingut anterior del p.i. ha estat esborrat */
  {
    esborra_node_arbreGen(primer_node);
    primer_node= NULL;
  }

  void plantar(const T &x)
  /* Pre: el p.i. és buit */
  /* Post: el p.i. té x com a arrel i zero fills */
  {
    primer_node= new node_arbreGen;
    primer_node->info= x;
    // No cal fer primer_node->seg = vector<node_arbreGen*>(0);
  } 

  void plantar(const T& x, vector<ArbreGen>& v){
  /* Pre: La longitud d'v és el nombre de fills del paràmetre implícit */
  /* Post: l'arbre té x com a arrel i els n elements de v com
     a fills */
    node_arbreGen* aux= new node_arbreGen;
    aux->info= x;
    int n=v.size();
    aux->seg = vector<node_arbreGen*>(n);
    for (int i=0; i<n; ++i) {
      aux->seg[i] = v[i].primer_node;
      v[i].primer_node = NULL;
    }
    primer_node= aux;
  }

  void afegir_fill(const ArbreGen &a)
  /* Pre: el p.i. i a no són buits */
  /* Post: el p.i. té un fill més que a l'inici, i aquest nou darrer fill
     és una còpia de l'arbre a */
  {
    //(primer_node->seg).push_back(copia_node_arbreGen(a.primer_node));
    (primer_node->seg).push_back(a.primer_node);
    a.primer_node = NULL;
  }


  void fill(const ArbreGen &a, int i)
  /* Pre: el p.i. és buit, a no és buit, i està entre 1 i el nombre de fills d'a */
  /* Post: el p.i és una còpia del fill i-èssim d'a */
  {
    primer_node= copia_node_arbreGen((a.primer_node)->seg[i-1]);
  }


  void fills(vector<ArbreGen> &v)
  /* Pre: el p.i. no és buit i li diem A, v és un vector buit */
  /* Post: el p.i. és buit, v passa a contenir els N fills de lâarbre A */
  {
    node_arbreGen* aux= primer_node;
    int n=aux->seg.size();
    v = vector<ArbreGen> (n);
    for (int i=0; i<n; ++i)
      v[i].primer_node = aux->seg[i];
    primer_node= NULL;
    delete aux;
  }

  //Consultores

  T arrel() const{
  /* Pre: L'arbre no és buit */
  /* Post: El resultat éss l'arrel de l'arbre */
    return primer_node->info;
  }

  bool es_buit() const{
  /* Pre: cert */
  /* Post: El resultat indica si l'arbre és buit o no */
    return (primer_node==NULL);
  }

  int nombre_fills() const{
  /* Pre: cert */
  /* Post: El nombre de fills de l'arbre */
    return (primer_node->seg).size();
  }

private:

  static node_arbreGen* copia_node_arbreGen(node_arbreGen* m){
    node_arbreGen* n;
    if (m==NULL) n=NULL;
    else {
      n = new node_arbreGen;
      n->info = m->info;
      int k = m->seg.size();
      n->seg = vector<node_arbreGen*>(k);
      for (int i=0; i<k; ++i)
	n->seg[i] = copia_node_arbreGen(m->seg[i]);
    }
    return n;
  }


  static void esborra_node_arbreGen(node_arbreGen* m){
    if (m != NULL) {
      for (int i=0; i<m->seg.size(); ++i)
	esborra_node_arbreGen(m->seg[i]);
      delete m;
    }
  }

};





#endif
